import os
import json
from fcache.cache import FileCache

cache = FileCache('snowmobile', flag='cs')


class Credentials:

    def __init__(self, config_file: str = 'snowflake_credentials.json',
                 conn_name: str = '', cache=cache) -> None:
        """Instantiates instances of the needed params to locate creds file.

        Args:
            config_file: Name of .json configuration file following the
            format of connection_credentials_SAMPLE.json.
            conn_name: Name of connection within json file to use - it will
            use first set of credentials in the file if no argument is passed.

        """
        self.cache = cache
        self.config_file = config_file
        self.conn_name = conn_name.lower()
        self.path_to_config = self.cache.get(r'path_to_config')

    def clear_cache(self) -> object:
        """Clears cached path to credentials file."""
        self.cache.clear()
        return self

    def cache_exists(self) -> bool:
        """Checks to see if a cached file path exists to a valid file."""
        try:
            return os.path.isfile(self.path_to_config)
        except:
            return False

    def cache_valid_for_config(self) -> bool:
        """Checks to see if the valid file path contains the config file."""
        try:
            return self.config_file == os.path.basename(self.path_to_config)
        except:
            return False

    def locate_config(self):
        """Find config file by traversing file system from bottom up."""
        for dirpath, dirnames, files in os.walk(os.path.expanduser('~'),
                                                topdown=False):
            if self.config_file in files:

                self.path_to_config = os.path.join(dirpath, self.config_file)
                break

            else:
                self.path_to_config = ''

        return self.path_to_config

    def get_creds_path(self) -> str:
        """Checks for cache existence and validates - traverses OS if not."""
        print("Locating credentials...")

        print("\t<1 of 2> Checking for cached path...")
        if self.cache_exists() and self.cache_valid_for_config():

            print(f"\t<2 of 2> Found cached path: {self.path_to_config}")

        else:

            print("\t<2 of 2> Cached path not found")
            print(f"\nLooking for {self.config_file} in local file system..")

            self.path_to_config = \
                self.locate_config()

            if self.path_to_config:
                print(f"\t<1 of 1> '{self.config_file}' found at: "
                      f"{self.path_to_config}")
            else:
                print(f"\t<1 of 1> Could not find config file"
                      f" {self.config_file} please double check the name of "
                      f"your configuration file or value passed in the"
                      f"'config_file' argument")

        return self.path_to_config

    def get(self) -> dict:
        """Locates creds file and parses out the specified set of credentials.

        Returns:
            Dictionary containing a specific set of Snowflake credentials

        """
        self.path_to_config = self.get_creds_path()
        self.cache['path_to_config'] = self.path_to_config

        try:
            with open(self.path_to_config) as c:
                temp = json.load(c)
                # print(temp)
                # self.all_creds = temp
                self.all_creds = \
                    {k.lower(): v for k, v in temp.items()}

        except IOError as e:
            print(e)

        print("\nImporting credentials...")
        if not self.conn_name:
            self.conn_name = next(iter(self.all_creds.keys()))
            print(f"\t<1 of 2> No explicit connection passed, fetching "
                  f"'{self.conn_name}' credentials by default")

        else:
            print(f"\t<1 of 2> Fetching user-specific 'conn_name="
                  f"'{self.conn_name}' from {self.config_file}")
            self.conn_name = self.conn_name


        # try:
        # self.conn_name = get_conn(self.all_creds, self.conn_name,
        #                           self.config_file)
        self.creds = self.all_creds[self.conn_name]

        if self.creds and self.conn_name:
            print(f"\t<2 of 2> Successfully imported credentials for "
                  f"'{self.conn_name}'")


        # except IOError as e:
        else:
            print(
                f"\t<2 of 2> Could not parse conn_name='{self.conn_name}' "
                f"from {self.config_file}\n\nPlease either \n\t\t- Specify "
                f"a different configuration file or connection name "
                f"\n\t\t- Verify the contents of the configuration file"
                f"\n\t\t- Run snowmobile.snowcreds.cache.clear()")
            # print(e)

        return self.creds



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

# sys.path.insert(0, os.path.abspath('.'))
print(os.path.abspath('source'))

# ----------/GEM/-----------------------
sys.path.insert(0, os.path.join(os.path.abspath('source').split('docs')[0],
                                'src', 'snowmobile'))

# print(f"GEM-os.path.abspath('.'):\n\t{os.path.abspath('source')}\n")
# print(f"GEM-os.path.abspath('..'):\n\t{os.path.abspath('')}\n")

# -- Project information -----------------------------------------------------

project = 'snowmobile'
copyright = '2020, Grant E Murray'
author = 'Grant E Murray'

# The full version, including alpha/beta/rc tags
import subprocess
release = \
    str((subprocess.check_output(['git', 'describe']).strip())).\
    split('-')[0].replace("'", '').replace('b', '')

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.intersphinx',
              'sphinx.ext.todo',
              'sphinx.ext.autosummary', 'sphinx.ext.viewcode',
              'sphinx.ext.coverage',
              'sphinx.ext.doctest', 'sphinx.ext.ifconfig',
              'sphinx.ext.mathjax',
              'sphinx.ext.napoleon', 'sphinx_rtd_theme', 'recommonmark',
              'autoapi.extension']

autoapi_type = 'python'

autoapi_dirs = ['../src/snowmobile', '../AUTHORS.rst',
                '../LICENSE.txt',
                '../CHANGELOG.rst', '../README.md']

autoapi_ignore = ['main.py', '__init__.py']

autoapi_python_class_content = 'both'

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'python'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'main.py', '__init__.py']

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'sphinx_rtd_theme'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# Napoleon settings
napoleon_google_docstring = True
napoleon_numpy_docstring = True
napoleon_include_init_with_doc = True
napoleon_include_private_with_doc = True
napoleon_include_special_with_doc = True
napoleon_use_admonition_for_examples = False
napoleon_use_admonition_for_notes = False
napoleon_use_admonition_for_references = False
napoleon_use_ivar = False
napoleon_use_param = False
napoleon_use_rtype = True
napoleon_use_keyword = True
napoleon_custom_sections = 'Attributes'

# Suffix of source filenames
source_suffix = ['.rst', '.md']

# Scaffolding paste from here down ============================================

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
# html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {}

# If false, no module index is generated.
# html_domain_indices = True

# If false, no index is generated.
# html_use_index = True

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
html_show_sphinx = False

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
html_show_copyright = False

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = None

# Output file base name for HTML help builder.
# htmlhelp_basename = 'cusersgem7318documentsgithubsnow_parser-doc'


# -- Options for LaTeX output --------------------------------------------------

# latex_elements = {
# # The paper size ('letterpaper' or 'a4paper').
# # 'papersize': 'letterpaper',
#
# # The font size ('10pt', '11pt' or '12pt').
# # 'pointsize': '10pt',
#
# # Additional stuff for the LaTeX preamble.
# # 'preamble': '',
# }

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
# latex_documents = [
#   ('index', 'user_guide.tex', u'C:\Users\GEM7318\Documents\Github\Snow-Parser Documentation',
#    u'Grant E Murray', 'manual'),
# ]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = ""

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# If true, show page references after internal links.
latex_show_pagerefs = True

# Highlighting for code blocks
pygments_style = 'sphinx'

# If true, show URL addresses after external links.
# latex_show_urls = False

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_domain_indices = True

# -- External mapping ------------------------------------------------------------
python_version = '.'.join(map(str, sys.version_info[0:2]))
intersphinx_mapping = {
    'sphinx': ('http://www.sphinx-doc.org/en/stable', None),
    'python': ('https://docs.python.org/' + python_version, None),
    'matplotlib': ('https://matplotlib.org', None),
    'numpy': ('https://docs.scipy.org/doc/numpy', None),
    'sklearn': ('http://scikit-learn.org/stable', None),
    'pandas': ('http://pandas.pydata.org/pandas-docs/stable', None),
    'scipy': ('https://docs.scipy.org/doc/scipy/reference', None),
}

# Adding so that __init__ will be documented - source is from link below:
# https://stackoverflow.com/questions/5599254/how-to-use-sphinxs-autodoc-to-document-a-classs-init-self-method

# def skip(app, what, name, obj, would_skip, options):
#     if name == "__init__":
#         return False
#     return would_skip
#
# def setup(app):
#     app.connect("autodoc-skip-member", skip)

autodoc_default_options = {
    'members': 'var1, var2',
    # 'member-order': 'bysource',
    'member-order': 'bysource',
    'special-members': '__init__',
    'undoc-members': True,
    'exclude-members': '__weakref__',
    'ignore-module-all': ['main.py'],
    'autoclass_content': 'both'
}


# To configure AutoStructify
def setup(app):
    from recommonmark.transform import AutoStructify
    app.add_config_value('recommonmark_config', {
        'auto_toc_tree_section': 'Contents',
        'enable_eval_rst': True,
        'enable_math': True,
        'enable_inline_math': True
    }, True)
    app.add_transform(AutoStructify)


# github_doc_root = 'https://github.com/GEM7318/Snowmobile/tree/master/docs'
# def setup(app):
#     from recommonmark.transform import AutoStructify
#     app.add_config_value('recommonmark_config', {
#             'url_resolver': lambda url: github_doc_root + url,
#             'auto_toc_tree_section': 'Contents',
#             }, True)
#     app.add_transform(AutoStructify)

# Turning off autoapi index since I manually added to TOC tree
autoapi_add_toctree_entry = False

# from sphinx.ext.autodoc import s



# -*- coding: utf-8 -*-

import pytest
from cusersgem7318documentsgithubsnowmobile.skeleton import fib

__author__ = "Grant E Murray"
__copyright__ = "Grant E Murray"
__license__ = "mit"


def test_fib():
    assert fib(1) == 1
    assert fib(2) == 1
    assert fib(7) == 13
    with pytest.raises(AssertionError):
        fib(-10)




from snowmobile import snowquery
import pandas as pd
import string
import os
import itertools
import csv
import datetime
import re


def standardize_col(col: str) -> str:
    """Standardize a column for Snowflake table.

    (1) Replaces spaces with underscores, trims leading & trailing
        underscores, forces upper-case
    (2) Replaces special characters with underscores
    (3) Reduces repeated special characters

    Args:
        col: A single string value of a column name
    Returns:
        A string that has been re-formatted/standardized for Snowflake
        standards

    """
    col = ((col.replace(' ', '_')).strip('_')).upper()  # 1

    invalid_punct = [punct for punct in string.punctuation if
                     punct != '_']  # 2
    repl_invalid_punct = str.maketrans(''.join(invalid_punct),
                                       '_' * len(invalid_punct))
    col = ''.join(col.translate(repl_invalid_punct))

    new_chars = []  # 3

    for k, v in itertools.groupby(col):
        v_list = list(v)

        if k in string.punctuation or k in string.whitespace:
            new_chars.append(k)

        else:
            new_chars.append(''.join(v_list))

    col = ''.join(new_chars)

    return col


def rename_cols_for_snowflake(df: pd.DataFrame) -> pd.DataFrame:
    """Renaming DataFrame columns for Snowflake table.

    Args:
        df: pd.DataFrame to be pushed to Snowflake
    Returns:
        pd.DataFrame with re-formatted column names and a *loaded_tmstmp*
        field added on the far right side
    """
    df['LOADED_TMSTMP'] = datetime.datetime.now()
    old_cols = list(df.columns)
    new_cols = [standardize_col(val) for val in df.columns]
    new_col_map = {k: v for k, v in zip(old_cols, new_cols)}
    df = df.rename(columns=new_col_map)
    return df


def get_ddl(df: pd.DataFrame, table_name: str) -> str:
    """Gets DDL for a table given a DataFrame.

    Args:
        df: pd.DataFrame to push to Snowflake.
        table_name: Name of table to load the DataFrame into.
    Returns:
        DDL to be executed to create table structure to load DataFrame into
        (for force-recreation of a table or loading into a table that
        doesn't previously exist)
    """
    final_ddl = \
        pd.io.sql.get_schema(df, table_name).replace('CREATE TABLE',
                                                     'CREATE OR REPLACE TABLE')
    return final_ddl


def check_information_schema(table_name: str,
                             snowflake: snowquery.Connector) -> list:
    """Checks information schema for existence of table & returns columns
    for comparison to local DataFrame if so.

    Args:
        table_name: Name of table to load the df into
        snowflake: snowquery.Connector object to execute statement with
    Returns:
        Columns of the table within database or an empty list if not
    """

    sql = f"""SELECT
                ORDINAL_POSITION
                ,COLUMN_NAME
            FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '{table_name}'
            ORDER BY 1 ASC"""

    validation_df = snowflake.execute_query(sql)

    try:
        table_cols = list(validation_df['COLUMN_NAME'])

    except:
        table_cols = []

    return table_cols


def compare_fields(df_cols: list, table_cols: list) -> int:
    """Returns match-count of column names in DataFrame compared to the table.

    Args:
        df_cols: Columns of the DataFrame to load
        table_cols: Columns of the table the DataFrame is being loaded into
    Returns:
        Count of matches between the table and the DataFrame's columns and
        will return a zero if the table does not exist at all
    """
    matched_list = []
    for i, (df_col, tbl_col) in enumerate(zip(df_cols, table_cols), start=1):

        if df_col == tbl_col:
            matched_list.append(i)

        else:
            pass

    matched_cnt = len(matched_list)

    return matched_cnt


def validate_table(df: pd.DataFrame, table_name: str,
                   snowflake: snowquery.Connector) -> tuple:
    """Analyzes count of matching columns to count of cols in df to load.

    Args:
        df: pd.DataFrame to load into Snowflake
        table_name: Name of table to load df into
    Returns:
        Tuple of boolean values indicating all possible combinations of a
        table existing (Y/N) and the columns of the table matching those in
        the DataFrame
    """
    table_cols = check_information_schema(table_name, snowflake)

    df_cols = list(df.columns)

    cnt_matches = compare_fields(df_cols, table_cols)

    if cnt_matches == len(df_cols) and cnt_matches == len(table_cols):
        outcome = True, True  # Table exists & fields match

    elif len(table_cols) != 0:
        outcome = True, False  # Table exists but fields don't match

    else:
        outcome = False, False  # Table doesn't exist

    return outcome


def verify_load(df: pd.DataFrame, snowflake: snowquery.Connector,
                table_name: str, force_recreate: bool =
                False) -> bool:
    """Performs pre-loading operations and comparisons to table in-warehouse.

    (1) Performs comparison of local DataFrame to in-warehouse table and
        creates/recreate the table if needed
    (2) Will not automatically recreate a table if it already exists but
        cannot append the contents of the DataFrame without modification

    Args:
        df: pd.DataFrame to push to Snowflake
        snowflake: snowquery.
        table_name: String representation of table name to load the df into
        force_recreate: Boolean value to indicating whether or not to
        force-recreation of table
    Returns:
        Boolean value indicating whether or not the loading process will
        occur successfully if continued based on the local to in-warehouse
        comparison
    """
    print(f"<validating load into {table_name}>")

    df = rename_cols_for_snowflake(df)

    table_exists, fields_match = validate_table(df, table_name,
                                                snowflake=snowflake)

    table_ddl = get_ddl(df, table_name)

    if not table_exists:
        print(
            f"\tTable: {table_name} created in absence of pre-existing "
            f"table\n")
        continue_load = True
        snowflake.execute_query(table_ddl)

    elif table_exists and fields_match and not force_recreate:
        print(
            f"\tTable: {table_name} already exists w/ matching field names\n"
            f"\t- continuing load and will append data to table\n")
        continue_load = True

    elif table_exists and fields_match and force_recreate:
        print(
            f"\tTable: {table_name} Already exists w/ matching field names "
            f"- Recreated by user w/ force_recreate=True\n")
        continue_load = True
        snowflake.execute_query(table_ddl)

    elif table_exists and not fields_match and not force_recreate:
        print(
            f"\tColumns in {table_name} don't match those in local DataFrame"
            f"\n\t- Use `force_recreate=True` to overwrite existing table\n")
        continue_load = False

    elif table_exists and not fields_match and force_recreate:
        print(
            f"\tTable: {table_name} columns don't match those in local "
            f"DataFrame \n- Force-recreated by user\n")
        continue_load = True
        snowflake.execute_query(table_ddl)

    else:
        print(
            f"\tUnknown error occured w/ load of {table_name} to Snowflake "
            f"\n\t- please check snowmobile.snowloader source codes")
        continue_load = False

    return continue_load


def remove_local(file_path: str, keep_local: bool = False) -> None:
    """Removes local copy of exported file post-loading.

    Args:
        file_path: Path to write local file to
        keep_local: Boolean value indicating whether or not to delete local
            file post-load
    Returns:
        None
    """
    if keep_local is False:
        os.remove(file_path)
        print(f"\n<Local copy of file deleted from {file_path}>")

    else:
        print(f"\n<Local copy of file saved in {file_path}>")

    return None


def df_to_snowflake(df: pd.DataFrame, table_name: str,
                    connector: snowquery.Connector = '',
                    force_recreate: bool = False, keep_local: bool = False,
                    output_location: str = os.getcwd(),
                    on_error: str = 'continue',
                    file_format: str = 'csv_gem7318') -> bool:
    """Loads DataFrame to a Snowflake table through a variety of operations.

    (1) Prepares DataFrame for load by standardizing column names
        and adding a *LOADED_TMSTMP* field to the far right side
    (2) Checks for existence of the table in Snowflake and compares
        structure of in-warehouse table to that of local DataFrame
    (3) Defaults to creating the table if it doesn't exist, appending to the
        table if it exists with matching field names/data types and will
        forgo loading the data/return a boolean value of False if
        otherwise; this can be over-ridden by passing ``force_recreate=True``
        when the function is called
    (4) Deletes local file written out to load into a staging table
    (5) Deletes the staging table after load is completed successfully
    (6) Returns a boolean value indicating whether or not the load was
        successful or not, intended for exception handling use when iterating
        through multiple files and appending to the same table such as:

        .. code-block:: python

            non_uniform_cols = {}
            for df_name, df in dfs_to_load.items():

                loaded = snowloader.df_to_snowflake(df, 'COMBINED_DFS')
                if not loaded:
                    non_uniform_cols[df_name] = df.columns

            print(f"Df(s) w/ Non-Uniform Columns:\\n\\t{non_uniform_cols.keys()}")

    Args:
        df: DataFrame to load to Snowflake
        table_name: Table name to load the data into
        connector: Pre-instantiated snowquery.Connector() instance with
            which to execute the load to Snowflake
        force_recreate: Boolean value indicating whether or not to recreate
            the table irrelevant of matching structure between local and DB
        keep_local: Boolean value indicating whether or not to keep the
            local .csv that is written out in the process
        output_location: Location to write out local .csv to; defaults to
            current working directory
        on_error: Query parameter for how to handle loading errors
        file_format: User-defined file_format within Snowflake
    Returns:
        Boolean value indicating whether or not load was successful

    """

    if not connector:
        connector = snowquery.Connector()

    continue_load = verify_load(snowflake=connector, df=df,
                                table_name=table_name,
                                force_recreate=force_recreate)

    if continue_load:

        # Committing changes to make sure table creation has gone through
        connector.commit()

        # File name for local copy
        file_name = f"{table_name}.csv"

        # Path to write to
        file_path = os.path.join(output_location, file_name)

        # Exporting csv to local drive
        df.to_csv(file_path, index=False, sep='|', header=False, quotechar='"',
                  quoting=csv.QUOTE_ALL)

        create_stage = \
            f"create or replace stage {table_name}_stage file_format " \
            f"= {file_format};"

        # Escaped path for put statement
        put_path = file_path.replace('\\', '\\\\')
        # put_path = re.escape(file_path)

        put_file = f"put 'file://{put_path}' @{table_name}_stage " \
                   f"auto_compress=true overwrite=true;"

        copy_into = f"copy into {table_name}\n" \
                    f"from @{table_name}_stage\n" \
                    f"on_error = '{on_error}';"

        drop_stage = f"drop stage {table_name}_stage;"

        statements = [create_stage, put_file, copy_into, drop_stage]

        for i, statement in enumerate(statements, start=1):

            try:
                result = connector.execute_query(statement)
                connector.commit()

                print(
                    f"\n<{i} of {len(statements)} completed>:\n\t"
                    f"{statement}\nResponse:\t")

                list_responses = \
                    [f"{' '.join(col.title().split('_'))}: {result.iat[0, i1]}"
                        for i1, col in enumerate(list(result.columns))]

                for response in list_responses:
                    print(f"\t{response}")

            except:
                print(f"\n<statement {i} of 4 failed>\n{statement}\n\n")
                connector.execute_query(statements[-1])
                break

        remove_local(file_path, keep_local)  # Defaults to delete local file

    else:
        i = 1
        pass

    if list(df.columns)[-1:] == ['LOADED_TMSTMP']:
        df.drop(columns=list(df.columns)[-1:], axis=1, inplace=True)
    else:
        pass

    if continue_load and i == 4:
        continue_load = True

    else:
        continue_load = False

    return continue_load





import snowflake.connector
from snowmobile import snowcreds as creds
from fcache.cache import FileCache

cache = FileCache('snowmobile', flag='cs')

class Connection(creds.Credentials):
    """Instantiate with inherited attributes from snowcreds.

    Args:
        config_file: Name of .json configuration file following the
            format of connection_credentials_SAMPLE.json.
        conn_name: Name of connection within json file to use - it will
            use first set of credentials in the file if no argument is
            passed.

    """
    def __init__(self, config_file: str = 'snowflake_credentials.json',
                 conn_name: str = '', cache=cache):
        super().__init__()

        self.config_file = config_file
        self.conn_name = conn_name
        self.cache = cache

    def get_conn(self) -> snowflake.connector:
        """Uses credentials to authenticate for statement execution.

        Returns:
            snowflake.connector.conn object

        """

        self.credentials = creds.Credentials(config_file=self.config_file,
                                             conn_name=self.conn_name,
                                             cache=self.cache).get()
        self.conn = snowflake.connector.connect(
            user=self.credentials["username"],
            password=self.credentials["password"],
            role=self.credentials["role"],
            account=self.credentials["account"],
            warehouse=self.credentials["warehouse"],
            database=self.credentials["database"],
            schema=self.credentials["schema"])

        return self.conn




import snowflake.connector as sf
import pandas as pd
from snowmobile import snowconn


class Connector(snowconn.Connection):
    """Primary Connection and Query Execution Class"""

    def __init__(self, config_file: str = 'snowflake_credentials.json',
                 conn_name: str = '') -> None:
        super().__init__(config_file, conn_name)

        self.conn = snowconn.Connection(config_file=self.config_file,
                                        conn_name=self.conn_name).get_conn()

    def execute_query(self, query: str, results: bool = True) -> \
            pd.DataFrame:
        """Execute commands & query data from the warehouse.

        Args:
            query: Raw SQL to execute.
            results: Boolean value indicating whether or not to return results
        Returns:
            Results from query in a Pandas DataFrame by default or None if
            ``results=False`` is passed when function is called.
        """
        self.query = query
        self.return_results = results

        if self.query:

            try:
                self.results = pd.read_sql(query, self.conn)

            except sf.errors.ProgrammingError as e:

                print(e)  # default error message

                print(f'Error {e.errno} ({e.sqlstate}): {e.msg} ('
                      f'{e.sfqid})')  # custom error message

                self.results = pd.DataFrame()

        if self.return_results:
            return self.results

        else:
            return None

    def disconnect(self) -> None:
        """Disconnect from connection with which Connect() was instantiated.

        Returns:
            None

        """
        self.conn.close()
        return None

    def commit(self) -> None:
        """Manually commits changes to database in instances when needed.

        Returns:
            None
        """
        self.conn.commit()

        return None

    def new(self) -> object:
        """Instantiates a new session for the same object.

        Returns:
            A snowquery.Connector() under the same set of credentials as the
            originally instantiated object but connected to a new session.
        """
        self.conn = snowconn.Connection(config_file=self.config_file,
                                        conn_name=self.conn_name).get_conn()

        return self
